package eu.hocke.roman.rquiz;

import android.content.DialogInterface;
import android.content.Intent;
import android.content.res.XmlResourceParser;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class QuizActivity extends AppCompatActivity {

    private static final int QUESTIONS_PER_QUIZ                 = 10;

    private static final int STATUS_UNINITIALIZED               = 0;
    private static final int STATUS_WAITING_FOR_ANSWER          = 1;
    private static final int STATUS_WAITING_FOR_CONFIRM         = 2;
    private static final int STATUS_SHOWING_RESULTS             = 3;

    private int status = STATUS_UNINITIALIZED;

    private String title = null;

    private List<QuizQuestion> questions = null;

    private Map<String, Button> answerButtons = null;

    private int currentQuestion = 0;

    private int correctAnswers = 0;

    private View.OnClickListener onClickListener = new View.OnClickListener() {

        public void onClick(View view) {

            answerButtonClicked(view);
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);

        Intent intent = getIntent();
        int quizId = intent.getIntExtra(QuizSelectActivity.EXTRA_QUIZ_ID, -1);
        if (quizId == -1)
            throw new IllegalArgumentException("quiz resource id not specified");

        try {

            questions = generateQuestions(quizId, QUESTIONS_PER_QUIZ);
        }
        catch (Exception e) {

            e.printStackTrace();
        }

        // TODO - restore state from savedInstanceState

        if (null == title)
            title = getResources().getString(R.string.title_activity_quiz);

        setContentView(R.layout.activity_quiz);
        setTitle(title);

        currentQuestion = 0;
        correctAnswers = 0;
        tryCurrentQuestion();
    }

    @Override
    public void onBackPressed() {

        if (status == STATUS_SHOWING_RESULTS) {

            super.onBackPressed();
            return;
        }

        AlertDialog.Builder alert = new AlertDialog.Builder(this);

        alert.setPositiveButton(R.string.quiz_back_button_quit, new DialogInterface.OnClickListener() {

            public void onClick(DialogInterface dialog, int which) {

                QuizActivity.this.finish();
            }
        });

        alert.setNegativeButton(R.string.quiz_back_button_stay, new DialogInterface.OnClickListener() {

            public void onClick(DialogInterface dialog, int which) {

                dialog.cancel();
            }
        });

        alert.setMessage(R.string.quiz_back_prompt);
        alert.show();
    }

    private List<QuizQuestion> generateQuestions(int resourceId, int count) throws IOException, XmlPullParserException {

        List<QuizQuestion> questions = loadAllQuestions(resourceId);
        Collections.shuffle(questions);

        while (questions.size() > count)
            questions.remove(questions.size() - 1);

        return questions;
    }

    public static String parseTitleFromXml(XmlPullParser parser) throws IOException, XmlPullParserException {

        while (parser.next() != XmlPullParser.END_DOCUMENT) {

            if (parser.getEventType() != XmlPullParser.START_TAG)
                continue;

            if (parser.getName().equals("quiz"))
                return parser.getAttributeValue(null, "title");
        }

        return null;
    }

    private List<QuizQuestion> loadAllQuestions(int resourceId) throws IOException, XmlPullParserException {

        XmlResourceParser parser = getResources().getXml(resourceId);
        Map<String, QuizOptionSet> optionSets = new HashMap<String, QuizOptionSet>();
        List<QuizQuestion> questions = new ArrayList<QuizQuestion>();

        while (parser.next() != XmlPullParser.END_DOCUMENT) {

            if (parser.getEventType() == XmlPullParser.START_TAG) {

                if (parser.getName().equals("optionset")) {

                    String optionSetId = parser.getAttributeValue(null, "id");
                    QuizOptionSet optionSet = QuizOptionSet.parseFromXml(parser);
                    optionSets.put(optionSetId, optionSet);
                }
                else
                if (parser.getName().equals("question")) {

                    QuizQuestion question = QuizQuestion.parseFromXml(parser, optionSets);
                    questions.add(question);
                }
                else
                if (parser.getName().equals("quiz")) {

                    title = parser.getAttributeValue(null, "title");
                }
            }
        }

        return questions;
    }

    private void startQuestion(QuizQuestion question) {

        cleanControls();
        updateCounters();

        // fill question text
        TextView questionText = (TextView)findViewById(R.id.quizDetailQuestion);
        questionText.setText(question.getQuestionText());
        findViewById(R.id.ScrlView).scrollTo(0, 0);

        // fill answering buttons
        answerButtons = question.createButtons(getApplicationContext(), onClickListener);
        LinearLayout parent = (LinearLayout)findViewById(R.id.quizDetailButtonGroup);
        for (Map.Entry entry : answerButtons.entrySet())
            parent.addView((View)entry.getValue());

        // set proper status
        status = STATUS_WAITING_FOR_ANSWER;
    }

    private void updateCounters() {

        ((TextView)findViewById(R.id.quizDetailCorrectValue)).setText(Integer.toString(correctAnswers));
        ((TextView)findViewById(R.id.quizDetailProgressValue)).setText(Integer.toString(currentQuestion + 1) + " / " + Integer.toString(questions.size()));
    }

    private void cleanControls() {

        LinearLayout buttons = (LinearLayout)findViewById(R.id.quizDetailButtonGroup);
        buttons.removeAllViews();

        TextView question = (TextView)findViewById(R.id.quizDetailQuestion);
        question.setText(null);

        answerButtons = null;
    }

    public void answerButtonClicked(View view) {

        if (status != STATUS_WAITING_FOR_ANSWER)
            return;

        boolean correct = false;
        Button button = (Button)view;
        QuizQuestion question = questions.get(currentQuestion);

        if (question == null)
            throw new IllegalArgumentException("answer for non-existent question");

        for (Map.Entry entry : answerButtons.entrySet()) {

            Button currentButton = (Button)entry.getValue();
            String id = (String)entry.getKey();

            if (question.isCorrectAnswer(id)) {

                currentButton.setTextColor(Color.GREEN);
                if (currentButton == view)
                    correct = true;
            }
            else
            if (button == currentButton)
                currentButton.setTextColor(Color.RED);
        }

        handleAnswer(correct);
    }

    public void tryCurrentQuestion() {

        if (currentQuestion >= questions.size()) {

            status = STATUS_SHOWING_RESULTS;
            updateCounters();
            cleanControls();

            setContentView(R.layout.activity_quiz_result);
            TextView score = (TextView)findViewById(R.id.quizResultScore);
            score.setText(Integer.toString(correctAnswers) + " / " + Integer.toString(questions.size()));

            // TODO - confirm button
            // TODO - high score
            // ...
        }
        else
            startQuestion(questions.get(currentQuestion));
    }

    public void handleAnswer(boolean correct) {

        if (correct)
            correctAnswers++;

        status = STATUS_WAITING_FOR_CONFIRM;
        new Handler().postDelayed(new Runnable() {

            public void run() {

                currentQuestion++;
                tryCurrentQuestion();

            }

        }, 1000);
    }
}
